"""

Python logging module

Loglevels: 
    
    LOGLEVEL_NONE, LOGLEVEL_NORMAL, LOGLEVEL_VERBOSE, LOGLEVEL_DEBUG

Format-Parameters:
    
    %C ..... class-name
    %D ..... program duration
    %F ..... function name
    %L ..... log-type
    %M ..... message
    %N ..... Line-number
    %T ..... current time

Pre-defined Formats:
    
     FMT_SHORT, FMT_MEDIUM, FMT_LONG, FMT_DEBUG

"""

# Logging levels

LOGLEVEL_NONE = 1 << 0
LOGLEVEL_NORMAL = 1 << 1
LOGLEVEL_VERBOSE = 1 << 2
LOGLEVEL_DEBUG = 1 << 3

# Pre-defined format strings

FMT_SHORT = "%M"
FMT_MEDIUM = "[ %C.%F ] %D: %M"
FMT_LONG = "%T %L %C [%F] - %M"
FMT_DEBUG = "%T (%D) %L %C [%F (%N)] - %M"

# Debug level constants

MSG_DEBUG = 1 << 0
MSG_WARN = 1 << 1
MSG_ERROR = 1 << 2
MSG_INFO = 1 << 3

# Boolean constants

TRUE = 1
FALSE = 0

# Color constants

BLACK = 30
RED = 31
GREEN = 32
YELLOW = 33
BLUE = 34
PURPLE = 35
AQUA = 36
WHITE = 37

LOG_MSG = {MSG_DEBUG: "DEBUG", MSG_WARN: "WARNING", MSG_ERROR: "ERROR", MSG_INFO: "INFO"}
LOG_COLORS = {MSG_DEBUG: [WHITE, BLACK, FALSE], MSG_WARN: [WHITE, BLACK, FALSE], MSG_ERROR: [WHITE, BLACK, TRUE], MSG_INFO: [WHITE, BLACK, FALSE]}
              
TIME_FORMAT = "%d.%m.%Y %H:%M:%S"

from time import time, strftime, localtime
from types import StringType, ClassType, InstanceType
from string import zfill, atoi
from re import sub

import sys
import traceback

# This is the main class for the logging module

class Category:

    # class initalization & customization
    def __init__(self, classid = None, target = None, formatstring = None, useansicodes = None):
        self.loglevel = LOGLEVEL_NORMAL
        self.timeinit = time()
        if (useansicodes == None):
            self.useansicodes = FALSE
        else:
            self.useansicodes = useansicodes
        if (classid == None):
            self.classname = "Main"
        else:
            if (type(classid) == StringType):
                self.classname = classid
            elif (type(classid) == ClassType):
                self.classname = classid.__name__
            elif (type(classid) == InstanceType):
                self.classname = classid.__class__.__name__
        if (formatstring == None):
            self.formatstr = FMT_LONG
        else:
            self.formatstr = formatstring
        self.target = target
        self.functionname = ""
        self.linenumber = -1
        
    def set_target(self, target):
        self.target = target

    def set_loglevel(self, loglevel):
        self.loglevel = loglevel

    def get_loglevel(self):
        return self.loglevel

    def set_formatstring(self, formatstring):
        self.formatstr = formatstring

    def set_use_ansi_codes(self, useansicodes):
        self.useansicodes = useansicodes

    def get_formatstring(self):
        return self.formatstr

    # Various message formats to print
    def debug(self, message):
        if (self.loglevel >= LOGLEVEL_DEBUG):
            self.showmessage(message, MSG_DEBUG)

    def warn(self, message):
        if (self.loglevel >= LOGLEVEL_VERBOSE):
            self.showmessage(message, MSG_WARN)

    def error(self, message):
        if (self.loglevel >= LOGLEVEL_NORMAL):
            self.showmessage(message, MSG_ERROR)

    def info(self, message):
        if (self.loglevel >= LOGLEVEL_NORMAL):
            self.showmessage(message, MSG_INFO)

    def tracestack(self):
        stack = traceback.extract_stack()
        self.linenumber = stack[-4][1]
        self.functionname = stack[-4][2]
        if (self.functionname == "?"):
            self.functionname = "Main"

    # Internal functions
    def showmessage(self, message, messagesource):
        currenttime = time()
        self.tracestack()
        timedifference = "%.3f" % (currenttime - self.timeinit)
        milliseconds = int(round((currenttime - long(currenttime)) * 1000))
        currenttime = strftime(TIME_FORMAT, localtime(currenttime))
        currenttime = "%s.%s" % (currenttime, zfill(milliseconds, 3))

        line = self.formatstr
        
        line = sub("%C", self.classname, line)
        line = sub("%D", timedifference, line)
        line = sub("%F", self.functionname, line)
        if (self.useansicodes == TRUE):
            line = sub("%L", self.ansi(LOG_MSG[messagesource], messagesource), line)
            line = sub("%M", self.ansi(message, messagesource), line)
        else:
            line = sub("%L", LOG_MSG[messagesource], line)
            line = sub("%M", message, line)
        line = sub("%N", str(self.linenumber), line)
        line = sub("%T", currenttime, line)

        if (self.target == None):
            sys.stdout.write("%s\n" % line)
        else:
            file = open(self.target, "a")
            file.write("%s\n" % line)
            file.close()

    # Convert text to text including ansi codes
    def ansi(self, text, messagesource):
        bold = LOG_COLORS[messagesource][2]
        fg = str(LOG_COLORS[messagesource][0])
        bg = LOG_COLORS[messagesource][1]
        if (bold == TRUE):
            fg = "%s;1" % fg
        bg = bg + 10
        text = "\033[%d;%sm%s\033[0m" % (bg, fg, text)
        return text
